#!/usr/bin/env python3
#
# Copyright (c) 2025 Stormux
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

"""Display Version plugin for Cthulhu."""

import logging
from cthulhu.plugin import Plugin, cthulhu_hookimpl
from cthulhu import cthulhuVersion

logger = logging.getLogger(__name__)

class DisplayVersion(Plugin):
    """Plugin that announces the current Cthulhu version."""
    
    def __init__(self, *args, **kwargs):
        """Initialize the plugin."""
        super().__init__(*args, **kwargs)
        logger.info("DisplayVersion plugin initialized")
        self._kb_binding = None
    
    @cthulhu_hookimpl
    def activate(self, plugin=None):
        """Activate the plugin."""
        # Skip if this activation call isn't for us
        if plugin is not None and plugin is not self:
            return
            
        try:
            logger.info("=== DisplayVersion plugin activation starting ===")
            logger.info(f"Plugin name: {self.name}")
            logger.info(f"App reference: {self.app}")
            
            # Also write to a debug file
            with open('/tmp/displayversion_debug.log', 'a') as f:
                f.write("=== DisplayVersion plugin activation starting ===\n")
                f.write(f"Plugin name: {self.name}\n")
                f.write(f"App reference: {self.app}\n")
            
            # Check if we have access to the API helper
            if not self.app:
                logger.error("DisplayVersion: No app reference available!")
                return
                
            api_helper = self.app.getAPIHelper()
            if not api_helper:
                logger.error("DisplayVersion: No API helper available!")
                return
                
            logger.info(f"API helper: {api_helper}")
            
            # Register keyboard shortcut
            gesture_string = 'kb:cthulhu+shift+v'
            logger.info(f"DisplayVersion: Attempting to register gesture: {gesture_string}")
            
            self._kb_binding = self.registerGestureByString(
                self.speakText, 
                f'Cthulhu screen reader version {cthulhuVersion.version}-{cthulhuVersion.codeName}', 
                gesture_string
            )
            
            logger.info(f"DisplayVersion: Registered keybinding result: {self._kb_binding}")
            if self._kb_binding:
                logger.info(f"Binding keysymstring: {self._kb_binding.keysymstring}")
                logger.info(f"Binding modifiers: {self._kb_binding.modifiers}")
                logger.info(f"Binding modifier_mask: {self._kb_binding.modifier_mask}")
                logger.info(f"Binding handler: {self._kb_binding.handler}")
                
                with open('/tmp/displayversion_debug.log', 'a') as f:
                    f.write(f"SUCCESS: Keybinding created!\n")
                    f.write(f"  keysymstring: {self._kb_binding.keysymstring}\n")
                    f.write(f"  modifiers: {self._kb_binding.modifiers}\n")
                    f.write(f"  modifier_mask: {self._kb_binding.modifier_mask}\n")
            else:
                logger.error("DisplayVersion: Failed to create keybinding!")
                with open('/tmp/displayversion_debug.log', 'a') as f:
                    f.write("ERROR: Failed to create keybinding!\n")
                
            logger.info("=== DisplayVersion plugin activation completed ===")
            with open('/tmp/displayversion_debug.log', 'a') as f:
                f.write("=== DisplayVersion plugin activation completed ===\n\n")
        except Exception as e:
            logger.error(f"Error activating DisplayVersion plugin: {e}")
            import traceback
            logger.error(traceback.format_exc())
    
    @cthulhu_hookimpl
    def deactivate(self, plugin=None):
        """Deactivate the plugin."""
        # Skip if this deactivation call isn't for us
        if plugin is not None and plugin is not self:
            return
            
        logger.info("Deactivating DisplayVersion plugin")
    
    def speakText(self, script=None, inputEvent=None):
        """Speak the Cthulhu version when shortcut is pressed."""
        try:
            logger.info("DisplayVersion plugin: speakText called")
            if self.app:
                state = self.app.getDynamicApiManager().getAPI('CthulhuState')
                if state.activeScript:
                    state.activeScript.presentMessage(
                        f'Cthulhu screen reader version {cthulhuVersion.version}-{cthulhuVersion.codeName}',
                        resetStyles=False
                    )
            return True
        except Exception as e:
            logger.error(f"Error in DisplayVersion speakText: {e}")
            return False
